//+------------------------------------------------------------------+
//|                                                       BBflat.mq4 |
//|                                                          by Raff |
//+------------------------------------------------------------------+

#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color1 Aqua
#property indicator_style1 2
#property indicator_color2 Red
#property indicator_color3 Red
#property indicator_color4 Blue

extern string TimeFrame = "Current time frame";
extern int period=9;
extern int shift=0;
extern int method=0;
extern int price=0;
extern double deviation=1.5;
extern bool   alertsOn         = true;
extern bool   alertsOnCurrent  = true;
extern bool   alertsMessage    = true;
extern bool   alertsSound      = false;
extern bool   alertsEmail      = false;
extern bool   Interpolate      = true;

//---- buffers
double ExtMapBuffer1[];
double ExtMapBuffer2[];
double ExtMapBuffer3[];
double ExtMapBuffer4[],cross[];

//
//
//
//
//

string indicatorFileName;
bool   returnBars;
int    timeFrame;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
   IndicatorBuffers(5);
   SetIndexBuffer(0,ExtMapBuffer1);
   SetIndexBuffer(1,ExtMapBuffer2);
   SetIndexBuffer(2,ExtMapBuffer3);
   SetIndexBuffer(3,ExtMapBuffer4);
   SetIndexBuffer(4,cross);
   
   //
   //
   //
   //
   //
   
   indicatorFileName = WindowExpertName();
   returnBars        = TimeFrame == "returnBars";     if (returnBars)     return(0);
   timeFrame         = stringToTimeFrame(TimeFrame);
   
   //
   //
   //
   //
   //

   IndicatorShortName(timeFrameToString(timeFrame)+" BBflat ");
   return(0);
}

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
           int limit=MathMin(Bars-counted_bars,Bars-1);
           if (returnBars) { ExtMapBuffer1[0] = limit+1; return(0); }
   
            if (timeFrame!=Period())
            {
               limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
               for (int i=limit; i>=0; i--)
               {
                   int y = iBarShift(NULL,timeFrame,Time[i]);               
                     ExtMapBuffer1[i] = 0;
                     ExtMapBuffer2[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",period,shift,method,price,deviation,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,1,y);
                     ExtMapBuffer3[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",period,shift,method,price,deviation,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,2,y);
                     ExtMapBuffer4[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",period,shift,method,price,deviation,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,3,y);
                     if (!Interpolate || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;

                     //
                     //
                     //
                     //
                     //
   
                     datetime time = iTime(NULL,timeFrame,y);
                     for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
                     for(int x = 1; x < n; x++)
                     {
  	                     ExtMapBuffer2[i+x] = ExtMapBuffer2[i] + (ExtMapBuffer2[i+n] - ExtMapBuffer2[i]) * x/n;
  	                     ExtMapBuffer3[i+x] = ExtMapBuffer3[i] + (ExtMapBuffer3[i+n] - ExtMapBuffer3[i]) * x/n;
  	                     ExtMapBuffer4[i+x] = ExtMapBuffer4[i] + (ExtMapBuffer4[i+n] - ExtMapBuffer4[i]) * x/n;
                     }  	            
               }
               return(0);
            }

   //
   //
   //
   //
   
   for (i=limit; i>=0; i--)
   {
	    double ima = iMA(NULL,0,period,shift,method,price,i);
       double std = deviation*iStdDev(NULL,0,period,shift,method,price,i);
		 ExtMapBuffer1[i] = 0;
       ExtMapBuffer2[i] = std;
       ExtMapBuffer3[i] =-std;
       ExtMapBuffer4[i] = Close[i]-ima;
       cross[i] = 0;
         if (ExtMapBuffer4[i]>ExtMapBuffer2[i]) cross[i] =  1;
         if (ExtMapBuffer4[i]<ExtMapBuffer3[i]) cross[i] = -1;
   }
   manageAlerts();
//----
   return(0);
  }

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1;
      if (cross[whichBar] != cross[whichBar+1])
      {
         if (cross[whichBar] ==  1) doAlert(whichBar,"up");
         if (cross[whichBar] == -1) doAlert(whichBar,"down");
      }
   }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," ",timeFrameToString(Period())," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," bbflat crossed deviation ",doWhat);
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol()," bbflat "),message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   StringToUpper(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}